﻿using ProjectStormtrooper.Drawnings;
using ProjectStormtrooper.Exceptions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectStormtrooper.CollectionGenericObjects;
/// <summary>
/// Параметрический набор объектов 
/// </summary>
/// <typeparam name="T">Параметр: ограничение - ссылочный тип</typeparam>
public class MassiveGenericObjects<T> : ICollectionGenericObjects<T>
    where T : class
{
    /// <summary>
    /// Массив объекта, которые храним
    /// </summary>
    private T?[] _collection;
    public int Count => _collection.Length;

    public int MaxCount
    {
        get
        {
            return _collection.Length;
        }
        set
        {
            if (value > 0)
            {
                if (_collection.Length > 0)
                {
                    Array.Resize(ref _collection, value);
                }
                else
                {
                    _collection = new T?[value];
                }
            }
        }
    }

    public CollectionType GetCollectionType => CollectionType.Massive;

    /// <summary>
    /// Конструктор
    /// </summary>
    public MassiveGenericObjects()
    {
        _collection = Array.Empty<T?>();
    }
    public T? Get(int position)
    {
        // проверка позиции
        if (position >= _collection.Length || position < 0) throw new PositionOutOfCollectionException(position);
        if (_collection[position] == null) throw new ObjectNotFoundException(position);
        return _collection[position];
    }

    public int Insert(T obj, IEqualityComparer<T?>? comparer = null)
    {
        if (comparer != null)
        {
            foreach(T? item in _collection)
            {
                if ((comparer as IEqualityComparer<DrawningStormtrooperBase>).Equals(obj as DrawningStormtrooperBase, item as DrawningStormtrooperBase))
                    throw new ObjectIsEqualException();
            }
        }
        // TODO вставка в свободное место набора
        int index = 0;
        while ( index < _collection.Length)
        {
            if (_collection[index] == null)
            {
                _collection[index] = obj;
                return index;
            }
            index++;
        }
        throw new CollectionOverflowException(Count);
    }

    public int Insert(T obj, int position, IEqualityComparer<T?>? comparer = null)
    {
        // TODO проверка позиции
        // TODO проверка, что элемент массива по этой позиции пустой, если нет, то
        //		ищется свободное место после этой позиции и идет вставка туда
        //		если нет после, ищем до
        // TODO вставка
        if (comparer != null)
        {
            foreach (T? item in _collection)
            {
                if ((comparer as IEqualityComparer<DrawningStormtrooperBase>).Equals(obj as DrawningStormtrooperBase, item as DrawningStormtrooperBase))
                    throw new ObjectIsEqualException();
            }
        }
        if (position >= _collection.Length || position < 0) throw new PositionOutOfCollectionException(position);
        if (_collection[position] == null)
        {
            _collection[position] = obj;
            return position;
        }
        int index = position + 1;
        while(index < _collection.Length)
        {
            if (_collection[index] == null)
            {
                _collection[index] = obj;
                return index;
            }
            index++;
        }
        index = position - 1;
        while ( index >= 0)
        {
            if (_collection[index] == null)
            {
                _collection[index] = obj;
                return index;
            }
            index--;
        }
        throw new CollectionOverflowException(Count);
    }

    public T? Remove(int position)
    {
        // TODO проверка позиции
        // TODO удаление объекта из массива, присвоив элементу массива значение null
        if (position >= _collection.Length || position < 0) throw new PositionOutOfCollectionException(position);
        if (_collection[position] == null) throw new ObjectNotFoundException(position);
        T temp = _collection[position];
        _collection[position] = null;
        return temp;
    }

    public IEnumerable<T?> GetItems()
    {
        for (int i = 0; i < _collection.Length; ++i)
        {
            yield return _collection[i];
        }
    }
    void ICollectionGenericObjects<T>.CollectionSort(IComparer<T?> comparer)
    {
        Array.Sort(_collection, comparer);
    }
}
